@@ -10,6 +10,7 @@ use constant IS_VMS => ( $^O eq 'VMS' );
 use TAP::Harness            ();
 use TAP::Parser::Aggregator ();
 
+use Config;
 use Exporter;
 
 # TODO: Emulate at least some of these
@@ -38,11 +39,11 @@ Test::Harness - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 3.01
+Version 3.02
 
 =cut
 
-$VERSION = '3.01';
+$VERSION = '3.02';
 
 # Backwards compatibility for exportable variable names.
 *verbose  = *Verbose;
@@ -107,6 +108,38 @@ one of the messages in the DIAGNOSTICS section.
 
 =cut
 
+sub _aggregate {
+    my ( $harness, $aggregate, @tests ) = @_;
+
+    # Don't propagate to our children
+    local $ENV{HARNESS_OPTIONS};
+
+    if (IS_VMS) {
+
+        # Jiggery pokery doesn't appear to work on VMS - so disable it
+        # pending investigation.
+        $harness->aggregate_tests( $aggregate, @tests );
+    }
+    else {
+        my $path_sep  = $Config{path_sep};
+        my $path_pat  = qr{$path_sep};
+        my @extra_inc = _filtered_inc();
+
+        my $previous = $ENV{PERL5LIB};
+        local $ENV{PERL5LIB};
+
+        if ($previous) {
+            push @extra_inc, split( $path_pat, $previous );
+        }
+
+        if (@extra_inc) {
+            $ENV{PERL5LIB} = join( $path_sep, @extra_inc );
+        }
+
+        $harness->aggregate_tests( $aggregate, @tests );
+    }
+}
+
 sub runtests {
     my @tests = @_;
 
@@ -116,7 +149,7 @@ sub runtests {
     my $harness   = _new_harness();
     my $aggregate = TAP::Parser::Aggregator->new();
 
-    $harness->aggregate_tests( $aggregate, @tests );
+    _aggregate( $harness, $aggregate, @tests );
 
     $harness->formatter->summary($aggregate);
 
@@ -176,7 +209,8 @@ sub _new_harness {
         }
     }
 
-    push @lib, _filtered_inc();
+    # Do things the old way on VMS...
+    push @lib, _filtered_inc() if IS_VMS;
 
     my $args = {
         timer      => $Timer,
@@ -186,13 +220,27 @@ sub _new_harness {
         verbosity  => $Verbose,
     };
 
+    if ( defined( my $env_opt = $ENV{HARNESS_OPTIONS} ) ) {
+        for my $opt ( split /:/, $env_opt ) {
+            if ( $opt =~ /^j(\d*)$/ ) {
+                $args->{jobs} = $1 || 9;
+            }
+            elsif ( $opt eq 'f' ) {
+                $args->{fork} = 1;
+            }
+            else {
+                die "Unknown HARNESS_OPTIONS item: $opt\n";
+            }
+        }
+    }
+
     return TAP::Harness->new($args);
 }
 
 # Get the parts of @INC which are changed from the stock list AND
 # preserve reordering of stock directories.
 sub _filtered_inc {
-    my @inc = grep { !ref } @INC; #28567
+    my @inc = grep { !ref } @INC;    #28567
 
     if (IS_VMS) {
 
@@ -289,7 +337,7 @@ sub execute_tests {
         }
     );
 
-    $harness->aggregate_tests( $aggregate, @{ $args{tests} } );
+    _aggregate( $harness, $aggregate, @{ $args{tests} } );
 
     $tot{bench} = $aggregate->elapsed;
     my @tests = $aggregate->descriptions;
@@ -454,6 +502,26 @@ If true, C<Test::Harness> will output the verbose results of running
 its tests.  Setting C<$Test::Harness::verbose> will override this,
 or you can use the C<-v> switch in the F<prove> utility.
 
+=item C<HARNESS_OPTIONS>
+
+Provide additional options to the harness. Currently supported options are:
+
+=over
+
+=item C<< j<n> >>
+
+Run <n> (default 9) parallel jobs.
+
+=item C<< f >>
+
+Use forked parallelism.
+
+=back
+
+Multiple options may be separated by colons:
+
+    HARNESS_OPTIONS=j9:f make test
+
 =back
 
 =head1 SEE ALSO
